import { strings } from '@material/select';
import { IContainer, AppTask, IAttrMapper, NodeObserverLocator } from 'aurelia';
import { FloatingLabelConfiguration } from '@aurelia2-mdc-web/floating-label';
import { LineRippleConfiguration } from '@aurelia2-mdc-web/line-ripple';
import { MdcSelect } from './mdc-select';
import { MdcSelectIcon } from './mdc-select-icon';
import { MdcSelectHelperText } from './mdc-select-helper-text/mdc-select-helper-text';
import { ListConfiguration } from '@aurelia2-mdc-web/list';
import { NotchedOutlineConfiguration } from '@aurelia2-mdc-web/notched-outline';
import { RippleConfiguration } from '@aurelia2-mdc-web/ripple';
import { MdcSelectValueObserver } from './mdc-select-value-observer';
import { MdcDefaultSelectConfiguration } from './mdc-default-select-configuration';
export { MdcSelect } from './mdc-select';
export { MdcDefaultSelectConfiguration };
let configured = false;
export const SelectConfiguration = {
    register(container) {
        if (!configured) {
            AppTask.creating(IContainer, c => {
                const attrMapper = c.get(IAttrMapper);
                const nodeObserverLocator = c.get(NodeObserverLocator);
                attrMapper.useTwoWay((el, property) => el.tagName === 'MDC-SELECT' ? property === 'value' : false);
                nodeObserverLocator.useConfig('MDC-SELECT', 'value', { events: [strings.CHANGE_EVENT], type: MdcSelectValueObserver });
            }).register(container);
            configured = true;
        }
        return container.register(MdcSelect, MdcSelectIcon, MdcSelectHelperText, ListConfiguration, FloatingLabelConfiguration, LineRippleConfiguration, NotchedOutlineConfiguration, RippleConfiguration);
    },
    customize(optionsProvider) {
        return {
            register(container) {
                const options = container.get(MdcDefaultSelectConfiguration);
                optionsProvider(options);
                return SelectConfiguration.register(container);
            },
        };
    }
};
//# sourceMappingURL=index.js.map