var MdcSelect_1;
import { __decorate, __metadata } from "tslib";
import { MdcComponent, booleanAttr } from '@aurelia2-mdc-web/base';
import { cssClasses, strings } from '@material/select';
import { inject, customElement, IPlatform, bindable } from 'aurelia';
import { mdcIconStrings } from './mdc-select-icon';
import { mdcHelperTextCssClasses } from './mdc-select-helper-text/mdc-select-helper-text';
import { MDCSelectFoundationAurelia } from './mdc-select-foundation-aurelia';
import { processContent, CustomElement, CustomAttribute } from '@aurelia/runtime-html';
import { MdcDefaultSelectConfiguration } from './mdc-default-select-configuration';
strings.CHANGE_EVENT = strings.CHANGE_EVENT.toLowerCase();
let selectId = 0;
/**
 * @selector mdc-select
 * @emits mdcselect:change | Emitted if user changed the value
 */
let MdcSelect = MdcSelect_1 = class MdcSelect extends MdcComponent {
    platform;
    defaultConfiguration;
    static processContent(node, platform) {
        const el = node;
        const leadingIcon = el.querySelector(`[${mdcIconStrings.ATTRIBUTE}]`);
        leadingIcon?.remove();
        const template = platform.document.createElement('template');
        template.setAttribute('au-slot', '');
        template.innerHTML = el.innerHTML;
        el.innerHTML = '';
        el.appendChild(template);
        // move icon to the slot - this allows omitting slot specification
        if (leadingIcon) {
            const div = platform.document.createElement('div');
            div.appendChild(leadingIcon);
            const iconTemplate = platform.document.createElement('template');
            iconTemplate.setAttribute('au-slot', 'leading-icon');
            iconTemplate.innerHTML = div.innerHTML;
            el.appendChild(iconTemplate);
        }
    }
    constructor(root, platform, defaultConfiguration) {
        super(root);
        this.platform = platform;
        this.defaultConfiguration = defaultConfiguration;
        defineMdcSelectElementApis(this.root);
        this.root.id = this.id;
    }
    id = `mdc-select-${++selectId}`;
    menu;
    selectAnchor;
    selectedText;
    menuElement;
    get items() {
        return this.menu.list_?.items;
    }
    leadingIcon;
    helperText;
    lineRipple;
    mdcLabel;
    outline;
    errors = new Map();
    /** Sets the select label */
    label;
    labelChanged() {
        this.platform.domWriteQueue.queueTask(() => this.foundation?.layout());
    }
    /** Styles the select as an outlined select */
    outlined;
    outlinedChanged() {
        this.platform.domWriteQueue.queueTask(() => this.foundation?.layout());
    }
    /** Makes the value required */
    required;
    requiredChanged() {
        if (this.required) {
            this.selectAnchor?.setAttribute('aria-required', 'true');
        }
        else {
            this.selectAnchor?.removeAttribute('aria-required');
        }
        this.foundation?.setRequired(this.required);
        this.platform.domWriteQueue.queueTask(() => this.foundation?.layout());
    }
    /** Enables/disables the select */
    disabled;
    disabledChanged() {
        if (this.disabled !== undefined) {
            this.foundation?.setDisabled(this.disabled);
        }
    }
    /** Hoists the select DOM to document.body */
    hoistToBody;
    /** Sets the select DOM position to fixed */
    fixed;
    /** Sets the margin between the select input and the dropdown */
    anchorMargin;
    /** Sets the select dropdown width to match content */
    naturalWidth;
    initialValue;
    bound() {
        if (typeof this.outlined !== 'boolean') {
            this.outlined = this.defaultConfiguration.outlined;
        }
    }
    get value() {
        if (this.foundation) {
            return this.foundation.getValue();
        }
        else {
            return this.initialValue;
        }
    }
    set value(value) {
        this.setValue(value);
    }
    setValue(value, skipNotify = false) {
        if (this.foundation) {
            this.foundation.setValue(value, skipNotify);
            this.foundation.layout();
        }
        else {
            this.initialValue = value;
        }
    }
    get valid() {
        return this.foundation?.isValid() ?? true;
    }
    set valid(value) {
        this.foundation?.setValid(value);
    }
    get selectedIndex() {
        return this.foundation.getSelectedIndex();
    }
    set selectedIndex(selectedIndex) {
        this.foundation?.setSelectedIndex(selectedIndex, /** closeMenu */ true);
    }
    addError(error) {
        this.errors.set(error, true);
        this.valid = false;
    }
    removeError(error) {
        this.errors.delete(error);
        this.valid = this.errors.size === 0;
    }
    renderErrors() {
        if (this.helperText) {
            this.helperText.errors = Array.from(this.errors.keys()).filter(x => x.message !== null).map(x => x.message);
        }
    }
    async attaching() {
        const nextSibling = this.root.nextElementSibling;
        if (nextSibling?.tagName === mdcHelperTextCssClasses.ROOT.toUpperCase()) {
            this.helperText = CustomElement.for(nextSibling).viewModel;
            await this.helperText.attachedPromise;
        }
    }
    beforeFoundationCreated() {
        const leadingIconEl = this.root.querySelector(`${strings.LEADING_ICON_SELECTOR}`);
        if (leadingIconEl) {
            this.leadingIcon = CustomAttribute.for(leadingIconEl, mdcIconStrings.ATTRIBUTE)?.viewModel;
        }
        this.menu.list_.singleSelection = true;
    }
    initialSyncWithDOM() {
        // set initial value without emitting change events
        this.foundation?.setValue(this.initialValue, true);
        this.foundation?.layout();
        this.errors = new Map();
        this.valid = true;
        this.labelChanged();
        this.disabledChanged();
        this.outlinedChanged();
        this.requiredChanged();
    }
    getDefaultFoundation() {
        // DO NOT INLINE this variable. For backward compatibility, foundations take a Partial<MDCFooAdapter>.
        // To ensure we don't accidentally omit any methods, we need a separate, strongly typed adapter variable.
        const adapter = {
            ...this.getSelectAdapterMethods(),
            ...this.getCommonAdapterMethods(),
            ...this.getOutlineAdapterMethods(),
            ...this.getLabelAdapterMethods(),
        };
        return new MDCSelectFoundationAurelia(adapter, this.getFoundationMap());
    }
    getSelectAdapterMethods() {
        return {
            setSelectedText: (text) => {
                this.selectedText.textContent = text;
            },
            isSelectAnchorFocused: () => document.activeElement === this.selectAnchor,
            getSelectAnchorAttr: (attr) => this.selectAnchor.getAttribute(attr),
            setSelectAnchorAttr: (attr, value) => {
                this.selectAnchor.setAttribute(attr, value);
            },
            removeSelectAnchorAttr: (attr) => {
                this.selectAnchor.removeAttribute(attr);
            },
            addMenuClass: (className) => {
                this.menuElement?.classList.add(className);
            },
            removeMenuClass: (className) => {
                this.menuElement?.classList.remove(className);
            },
            openMenu: () => {
                this.menu.open = true;
                if ((this.hoistToBody || this.fixed) && !this.naturalWidth) {
                    this.menu.root.style.minWidth = this.menu.root.style.maxWidth = `${this.root.clientWidth}px`;
                }
            },
            closeMenu: () => { this.menu.open = false; },
            getAnchorElement: () => this.root.querySelector(strings.SELECT_ANCHOR_SELECTOR),
            setMenuAnchorElement: (anchorEl) => {
                this.menu.anchor = anchorEl;
            },
            setMenuAnchorCorner: (anchorCorner) => {
                this.menu.setAnchorCorner(anchorCorner);
            },
            setMenuWrapFocus: (wrapFocus) => {
                this.menu.wrapFocus = wrapFocus;
            },
            getSelectedIndex: () => {
                const index = this.menu.selectedIndex;
                return index instanceof Array ? index[0] : index;
            },
            setSelectedIndex: (index) => {
                this.menu.selectedIndex = index;
            },
            removeAttributeAtIndex: (index, attributeName) => {
                this.menu.items[index].removeAttribute(attributeName);
            },
            focusMenuItemAtIndex: (index) => {
                this.menu.items[index].focus();
            },
            getMenuItemCount: () => this.menu.items.length,
            getMenuItemValues: () => this.menu.items.map(x => CustomElement.for(x).viewModel.value),
            getMenuItemTextAtIndex: (index) => this.menu.getPrimaryTextAtIndex(index),
            isTypeaheadInProgress: () => this.menu.typeaheadInProgress,
            typeaheadMatchItem: (nextChar, startingIndex) => this.menu.typeaheadMatchItem(nextChar, startingIndex),
        };
    }
    getCommonAdapterMethods() {
        return {
            addClass: (className) => {
                this.root.classList.add(className);
            },
            removeClass: (className) => {
                this.root.classList.remove(className);
            },
            hasClass: (className) => this.root.classList.contains(className),
            setRippleCenter: (normalizedX) => this.lineRipple?.setRippleCenter(normalizedX),
            activateBottomLine: () => this.lineRipple?.activate(),
            deactivateBottomLine: () => this.lineRipple?.deactivate(),
            notifyChange: (value) => {
                const index = this.selectedIndex;
                this.emit(strings.CHANGE_EVENT, { value, index }, true /* shouldBubble  */);
                this.emit('change', { value, index }, true /* shouldBubble  */);
            },
        };
    }
    getOutlineAdapterMethods() {
        return {
            hasOutline: () => Boolean(this.outline),
            notchOutline: (labelWidth) => this.outline?.notch(labelWidth),
            closeOutline: () => this.outline?.closeNotch(),
        };
    }
    getLabelAdapterMethods() {
        return {
            hasLabel: () => !!this.mdcLabel,
            floatLabel: (shouldFloat) => this.mdcLabel?.float(shouldFloat),
            getLabelWidth: () => this.mdcLabel ? this.mdcLabel.getWidth() : 0,
            setLabelRequired: (isRequired) => this.mdcLabel?.setRequired(isRequired),
        };
    }
    handleChange() {
        this.foundation?.handleChange();
        this.emit('change', {}, true);
    }
    handleFocus() {
        this.foundation?.handleFocus();
    }
    handleBlur() {
        this.foundation?.handleBlur();
        // if class is set it means the menu is open,
        // do not emit blur since "conceptually" the element is still active
        if (!this.root.classList.contains(cssClasses.FOCUSED)) {
            this.emit('blur', {}, true);
        }
    }
    handleClick(evt) {
        this.selectAnchor.focus();
        this.foundation?.handleClick(this.getNormalizedXCoordinate(evt));
    }
    handleKeydown(evt) {
        this.foundation?.handleKeydown(evt);
        return true;
    }
    handleMenuItemAction(evt) {
        this.foundation?.handleMenuItemAction(evt.detail.index);
    }
    handleMenuOpened() {
        this.foundation?.handleMenuOpened();
    }
    handleMenuClosed() {
        this.foundation?.handleMenuClosed();
        if (!this.root.classList.contains(cssClasses.FOCUSED)) {
            this.emit('blur', {}, true);
        }
    }
    handleItemsChanged() {
        this.foundation?.layoutOptions();
        this.foundation?.layout();
    }
    focus() {
        this.selectAnchor.focus();
    }
    blur() {
        this.selectAnchor.blur();
    }
    /**
     * @hidden
     * Calculates where the line ripple should start based on the x coordinate within the component.
     */
    getNormalizedXCoordinate(evt) {
        const targetClientRect = evt.target.getBoundingClientRect();
        const xCoordinate = this.isTouchEvent(evt) ? evt.touches[0].clientX : evt.clientX;
        return xCoordinate - targetClientRect.left;
    }
    isTouchEvent(evt) {
        return Boolean(evt.touches);
    }
    /**
     * @hidden
     * Returns a map of all subcomponents to subfoundations.
     */
    getFoundationMap() {
        return {
            helperText: this.helperText?.foundation,
            leadingIcon: this.leadingIcon?.foundation
        };
    }
};
__decorate([
    bindable,
    __metadata("design:type", String)
], MdcSelect.prototype, "label", void 0);
__decorate([
    bindable({ set: booleanAttr }),
    __metadata("design:type", Boolean)
], MdcSelect.prototype, "outlined", void 0);
__decorate([
    bindable({ set: booleanAttr }),
    __metadata("design:type", Boolean)
], MdcSelect.prototype, "required", void 0);
__decorate([
    bindable({ set: booleanAttr }),
    __metadata("design:type", Boolean)
], MdcSelect.prototype, "disabled", void 0);
__decorate([
    bindable({ set: booleanAttr, mode: 1 /* BindingMode.oneTime */ }),
    __metadata("design:type", Boolean)
], MdcSelect.prototype, "hoistToBody", void 0);
__decorate([
    bindable({ set: booleanAttr, mode: 1 /* BindingMode.oneTime */ }),
    __metadata("design:type", Boolean)
], MdcSelect.prototype, "fixed", void 0);
__decorate([
    bindable,
    __metadata("design:type", Object)
], MdcSelect.prototype, "anchorMargin", void 0);
__decorate([
    bindable({ set: booleanAttr }),
    __metadata("design:type", Boolean)
], MdcSelect.prototype, "naturalWidth", void 0);
MdcSelect = MdcSelect_1 = __decorate([
    inject(Element, IPlatform, MdcDefaultSelectConfiguration),
    customElement('mdc-select'),
    processContent(MdcSelect_1.processContent),
    __metadata("design:paramtypes", [HTMLElement, Object, MdcDefaultSelectConfiguration])
], MdcSelect);
export { MdcSelect };
function defineMdcSelectElementApis(element) {
    Object.defineProperties(element, {
        value: {
            get() {
                return CustomElement.for(this).viewModel.value;
            },
            set(value) {
                // aurelia binding converts "undefined" and "null" into empty string
                // this does not translate well into "empty" menu items when several selects are bound to the same field
                CustomElement.for(this).viewModel.value = value === '' ? undefined : value;
            },
            configurable: true
        },
        options: {
            get() {
                return CustomElement.for(this).viewModel.root.querySelectorAll('.mdc-list-item');
            },
            configurable: true
        },
        selectedIndex: {
            get() {
                return CustomElement.for(this).viewModel.selectedIndex;
            },
            set(value) {
                CustomElement.for(this).viewModel.selectedIndex = value;
            },
            configurable: true
        },
        valid: {
            get() {
                return CustomElement.for(this).viewModel.valid;
            },
            set(value) {
                CustomElement.for(this).viewModel.valid = value;
            },
            configurable: true
        },
        addError: {
            value(error) {
                CustomElement.for(this).viewModel.addError(error);
            },
            configurable: true
        },
        removeError: {
            value(error) {
                CustomElement.for(this).viewModel.removeError(error);
            },
            configurable: true
        },
        renderErrors: {
            value() {
                CustomElement.for(this).viewModel.renderErrors();
            },
            configurable: true
        },
        focus: {
            value() {
                CustomElement.for(this).viewModel.focus();
            },
            configurable: true
        },
        blur: {
            value() {
                CustomElement.for(this).viewModel.blur();
            },
            configurable: true
        }
    });
}
//# sourceMappingURL=mdc-select.js.map