import { CollectionKind, AccessorType } from '@aurelia/runtime';
import type { ICollectionObserver, IndexMap, IObserver, IObserverLocator, ISubscriber, ISubscriberCollection } from '@aurelia/runtime';
import { INode } from '@aurelia/runtime-html';
import { IMdcSelectElement } from './mdc-select';
export interface IOptionElement extends HTMLOptionElement {
    model?: unknown;
}
export interface MdcSelectValueObserver extends ISubscriberCollection {
}
export interface INodeObserverConfigBase {
    /**
     * Indicates the list of events can be used to observe a particular property
     */
    readonly events: string[];
    /**
     * Indicates whether this property is readonly, so observer wont attempt to assign value
     * example: input.files
     */
    readonly readonly?: boolean;
    /**
     * A default value to assign to the corresponding property if the incoming value is null/undefined
     */
    readonly default?: unknown;
}
export interface INodeObserver extends IObserver {
    /**
     * Instruct this node observer event observation behavior
     */
    useConfig(config: INodeObserverConfigBase): void;
}
export declare class MdcSelectValueObserver implements INodeObserver {
    currentValue: unknown;
    oldValue: unknown;
    readonly obj: IMdcSelectElement;
    config: INodeObserverConfigBase;
    hasChanges: boolean;
    type: AccessorType;
    arrayObserver?: ICollectionObserver<CollectionKind.array>;
    nodeObserver?: MutationObserver;
    private observing;
    private listened;
    constructor(obj: INode, _key: PropertyKey, config: INodeObserverConfigBase, _: IObserverLocator);
    optionsWereSet: boolean;
    setElementValue(skipNotify: boolean): void;
    getValue(): unknown;
    setValue(newValue: unknown): void;
    flushChanges(): void;
    handleCollectionChange(): void;
    notify(): void;
    handleEvent(): void;
    synchronizeOptions(_indexMap?: IndexMap): void;
    synchronizeValue(): boolean;
    private start;
    private stop;
    handleNodeChange(records: MutationRecord[]): void;
    subscribe(subscriber: ISubscriber): void;
    unsubscribe(subscriber: ISubscriber): void;
    useConfig(config: INodeObserverConfigBase): void;
}
