import { MdcComponent, IValidatedElement, IError } from '@aurelia2-mdc-web/base';
import { INode, IPlatform } from 'aurelia';
import { MdcMenu } from '@aurelia2-mdc-web/menu';
import { MDCMenuItemEvent } from '@material/menu';
import { MdcListItem } from '@aurelia2-mdc-web/list';
import { MDCSelectFoundationAurelia } from './mdc-select-foundation-aurelia';
import { MDCMenuDistance } from '@material/menu-surface';
import { MdcDefaultSelectConfiguration } from './mdc-default-select-configuration';
/**
 * @selector mdc-select
 * @emits mdcselect:change | Emitted if user changed the value
 */
export declare class MdcSelect extends MdcComponent<MDCSelectFoundationAurelia> {
    private platform;
    private defaultConfiguration;
    static processContent(node: INode, platform: IPlatform): void;
    constructor(root: HTMLElement, platform: IPlatform, defaultConfiguration: MdcDefaultSelectConfiguration);
    id: string;
    menu: MdcMenu;
    private selectAnchor;
    private selectedText;
    private menuElement?;
    get items(): MdcListItem[] | undefined;
    private leadingIcon?;
    private helperText?;
    private lineRipple?;
    private mdcLabel;
    private outline?;
    errors: Map<IError, boolean>;
    /** Sets the select label */
    label: string;
    labelChanged(): void;
    /** Styles the select as an outlined select */
    outlined?: boolean;
    outlinedChanged(): void;
    /** Makes the value required */
    required: boolean;
    requiredChanged(): void;
    /** Enables/disables the select */
    disabled?: boolean;
    disabledChanged(): void;
    /** Hoists the select DOM to document.body */
    hoistToBody: boolean;
    /** Sets the select DOM position to fixed */
    fixed: boolean;
    /** Sets the margin between the select input and the dropdown */
    anchorMargin: Partial<MDCMenuDistance>;
    /** Sets the select dropdown width to match content */
    naturalWidth: boolean;
    private initialValue;
    bound(): void;
    get value(): unknown;
    set value(value: unknown);
    setValue(value: unknown, skipNotify?: boolean): void;
    get valid(): boolean;
    set valid(value: boolean);
    get selectedIndex(): number;
    set selectedIndex(selectedIndex: number);
    addError(error: IError): void;
    removeError(error: IError): void;
    renderErrors(): void;
    attaching(): Promise<void>;
    beforeFoundationCreated(): void;
    initialSyncWithDOM(): void;
    getDefaultFoundation(): MDCSelectFoundationAurelia;
    private getSelectAdapterMethods;
    private getCommonAdapterMethods;
    private getOutlineAdapterMethods;
    private getLabelAdapterMethods;
    handleChange(): void;
    handleFocus(): void;
    handleBlur(): void;
    handleClick(evt: MouseEvent): void;
    handleKeydown(evt: KeyboardEvent): boolean;
    handleMenuItemAction(evt: MDCMenuItemEvent): void;
    handleMenuOpened(): void;
    handleMenuClosed(): void;
    handleItemsChanged(): void;
    focus(): void;
    blur(): void;
    /**
     * @hidden
     * Calculates where the line ripple should start based on the x coordinate within the component.
     */
    private getNormalizedXCoordinate;
    private isTouchEvent;
    /**
     * @hidden
     * Returns a map of all subcomponents to subfoundations.
     */
    private getFoundationMap;
}
/** @hidden */
export interface IMdcSelectElement extends IValidatedElement {
    $au: {
        'au:resource:custom-element': {
            viewModel: MdcSelect;
        };
    };
    value: unknown;
}
